#include <avr/io.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <compat/deprecated.h>

#define F_CPU 8000000UL		// 8 MHz Takt
#define Takt 8000000UL		// Takt fr USART
#define Baud 19200UL		// Baud fr USART
#include "myusart.h"		// USART Libary einbinden
#include "myadc.h"			// ADC Libary einbinden

unsigned char Puffer[3];
unsigned char counter=0;
unsigned char zeichen,startbit=0,stoppbit=0;
unsigned char impulse_links=0,impulse_rechts=0;
unsigned int zeit=0;
		
ISR(USART_RXC_vect)								// USART Empfnger Interrupt-Service-Routine
{
	zeichen = get_byte();
	
	if(startbit==2 && !(zeichen==41))
	{
		Puffer[counter]=zeichen;
		counter++;
	}
	
	if(zeichen==40 && startbit<2)startbit++;
	if(zeichen==41 && stoppbit<2)stoppbit++;
	
	if(stoppbit==2)
	{
		impulse_links=0;
		impulse_rechts=0;
		OCR1AL=Puffer[1];
		OCR1BL=Puffer[2];
		if(Puffer[0]==3)Robot_vor();
		if(Puffer[0]==0)Robot_zurueck();
		if(Puffer[0]==1)Robot_rechts();
		if(Puffer[0]==2)Robot_links();
		if(Puffer[0] >= 0 && Puffer[0] <= 3)
		{
			wdt_enable(4);
			wdt_reset();
		}
		if(Puffer[1]==0 && Puffer[2]==0) Robot_stopp();
			
		counter=0;
		startbit=0;
		stoppbit=0;
	}
}

ISR(INT0_vect)
{
	impulse_links++;
}

ISR(INT1_vect)
{
	impulse_rechts++;
	/*
	OCR1AL=120;
	OCR1BL=120;
	if(impulse_rechts<=100)Robot_vor();
	if(impulse_rechts>=100)Robot_zurueck();
	if(impulse_rechts==200)
	{
		Robot_stopp();
		impulse_rechts=0;
	}
	*/
}

ISR (TIMER2_COMP_vect)
{
		zeit++;
		if(zeit==50)
		{
			zeit=0;
			//send_byte(impulse_links);
			send_byte(impulse_rechts);
		}
}

int Robot_vor(void)
{
	PORTB |= (1 << PB0);		// Linke Kette vor
	PORTD |= (1 << PD7);		// Rechte Kette vor
	return(0);
}

int Robot_links(void)
{
	PORTB |= (1 << PB0);		// Linke Kette zurck
	PORTD &= ~(1 << PD7);		// Rechte Kette vor
}

int Robot_rechts(void)
{
	PORTB &= ~(1 << PB0);		// Linke Kette vor
	PORTD |= (1 << PD7);		// Rechte Kette zurck
}

int Robot_zurueck(void)
{
	PORTB &= ~(1 << PB0);		// Linke Kette zurck
	PORTD &= ~(1 << PD7);		// Rechte Kette zurck
	return(0);
}

int Robot_stopp(void)
{
	OCR1AL=0;					// Geschwindigkeit Linke Kette = 0
	OCR1BL=0;					// Geschwindigkeit Rechte Kette = 0
	wdt_disable();
}
					
int main(void)
{
	DDRB = 0xff; 				// PortB als Ausgang
	DDRD = 0xff; 				// PortD als Ausgang
	DDRC = 0xff; 				// PortC als Ausgang
	DDRD &= ~(1 << PD2);
	DDRD &= ~(1 << PD3);
	//PORTD |= (1 << PD2);		// Pullup aktivieren
	//PORTD |= (1 << PD3);		// Pullup aktivieren
	
	
	TCCR2 = (1<<WGM21); 		// CTC Modus
	TCCR2 = (1<<CS22);  		// Prescaler 64
	OCR2 = 125-1;				// Vergleichswert
	TIMSK |= (1<<OCIE2);		// Interrupt aktivieren

	TCCR1A |=  (1 << WGM10);  	// PWM Phasenkorrekt 8 Bit Kanal 1
	TCCR1A &= ~(1 << WGM11);  	// PWM Phasenkorrekt 8 Bit Kanal 1
	TCCR1B &= ~(1 << WGM12);  	// PWM Phasenkorrekt 8 Bit Kanal 2
	TCCR1B &= ~(1 << WGM13);  	// PWM Phasenkorrekt 8 Bit Kanal 2

	TCCR1B |=  (1 << CS10);	// Systemtakt
	TCCR1B &= ~(1 << CS11);	// Systemtakt

	TCCR1A |= (1 << COM1A0);	// Invertierende PWM Kanal 1
	TCCR1A |= (1 << COM1A1);	// Invertierende PWM kanal 1

	TCCR1A |= (1 << COM1B0);	// Invertierende PWM Kanal 2
	TCCR1A |= (1 << COM1B1);	// Invertierende PWM Kanal 2

	init_usart();				// USART initialisieren
	sbi(UCSRB, RXCIE);			// Empfngerinterrupt frei
	
	MCUCR |= (1<<ISC11);		// INT0 fallende Flanke
	MCUCR |= (1<<ISC01);		// INT1 fallende Flanke
	
	GICR |= (1<<INT0);			// INT0 frei
	GICR |= (1<<INT1);			// INT1 frei
	
	sei();						// alle Interrupts global frei
	Robot_stopp();				// Roboter stoppen
	wdt_disable();
	
	while(1)					// Programmschleife
	{
	}
	return(0);
}
